<?php

namespace App\Console;

use App\Service\DatabaseService;
use Symfony\Component\Console\Attribute\AsCommand;
use Symfony\Component\Console\Command\Command;
use Symfony\Component\Console\Input\InputArgument;
use Symfony\Component\Console\Input\InputInterface;
use Symfony\Component\Console\Output\OutputInterface;
use Symfony\Component\Console\Question\ChoiceQuestion;
use Symfony\Component\Console\Style\SymfonyStyle;

#[AsCommand('app:permission:list')]
class ListPermissionsCommand extends Command
{
    public function __construct(
        private readonly DatabaseService $db,
    )
    {
        parent::__construct();
    }

    public function configure(): void
    {
        $this->addArgument('name', InputArgument::OPTIONAL, 'User name');
        $this->addArgument('host', InputArgument::OPTIONAL, 'User host');
    }

    protected function execute(InputInterface $input, OutputInterface $output): int
    {
        $style = new SymfonyStyle($input, $output);

        $name = $input->getArgument('name');
        $host = $input->getArgument('host');

        if (!$name) {
            $question = $this->getHelper('question');
            $selectQuestion = new ChoiceQuestion('User name: ', array_map(
                    fn($user) => $user['name'],
                    $this->db->listUsers())
            );
            $name = $question->ask($input, $output, $selectQuestion);
        }

        $permissions = $this->db->listPermissions($name, $host);

        $table = $style
            ->createTable()
            ->setHeaders(['Permission'])
            ->setHeaderTitle(sprintf('List of users for %s@%s', $name, $host))
        ;
        foreach ($permissions as $permission) {
            $table->addRow([$permission]);
        }
        $table->render();

        return Command::SUCCESS;
    }
}